searchData={"items":[{"type":"module","title":"ExCmd","doc":"ExCmd is an Elixir library to run and communicate with external\nprograms with back-pressure.","ref":"ExCmd.html"},{"type":"function","title":"ExCmd.stream!/2","doc":"Runs the given command with arguments and return an Enumerable to\nread command output.\n\nFirst parameter must be a list containing command with\narguments. example: `[\"cat\", \"file.txt\"]`.","ref":"ExCmd.html#stream!/2"},{"type":"function","title":"Optional - ExCmd.stream!/2","doc":"* `input` - Input can be binary, iodata, `Enumerable` (list, stream)\n     or a function which accepts `Collectable`.\n\n    * Binary\n      ```\n      # binary\n      ExCmd.stream!(~w(base64), input: \"Hello Wolrd\")\n      ```\n\n    * Enumerable or iodata\n\n      ```\n      # list\n      ExCmd.stream!(~w(base64), input: [\"hello\", \"world\"])\n\n      # iodata\n      ExCmd.stream!(~w(cat), input: [\"He\", [\"llo\"], [' ', ?W], ['or', \"ld\"]])\n\n      # stream\n      ExCmd.stream!(~w(cat), input: File.stream!(\"log.txt\", [], 65536))\n      ```\n\n    * Collectable:\n\n      If the input in a function with arity 1, ExCmd will call that\n      function with a `Collectable` as the argument. The function\n      must *push* input to this collectable. Return value of the\n      function is ignored.\n\n      ```\n      ExCmd.stream!(~w(cat), input: fn sink ->\n        Enum.into(1..100, sink, &to_string/1)\n      end)\n      ```\n\n  * `exit_timeout` - Time to wait for external program to exit.\n    After writing all of the input we close the stdin stream and\n    wait for external program to finish and exit. If program does\n    not exit within `exit_timeout` an error will be raised and\n    subsequently program will be killed in background. Defaults\n    timeout is `:infinity`.\n\nAll other options are passed to `ExCmd.Process.start_link/2`","ref":"ExCmd.html#stream!/2-optional"},{"type":"function","title":"Examples - ExCmd.stream!/2","doc":"```\nExCmd.stream!(~w(ffmpeg -i pipe:0 -f mp3 pipe:1), input: File.stream!(\"music_video.mkv\", [], 65336))\n|> Stream.into(File.stream!(\"music.mp3\"))\n|> Stream.run()\n```\n\nWith input as binary\n\n```\niex> ExCmd.stream!(~w(cat), input: \"Hello World\")\n...> |> Enum.into(\"\")\n\"Hello World\"\n```\n\n```\niex> ExCmd.stream!(~w(base64), input: <<1, 2, 3, 4, 5>>)\n...> |> Enum.into(\"\")\n\"AQIDBAU=\n\"\n```\n\nWith input as list\n\n```\niex> ExCmd.stream!(~w(cat), input: [\"Hello \", \"World\"])\n...> |> Enum.into(\"\")\n\"Hello World\"\n```\n\nWith input as iodata\n\n```\niex> ExCmd.stream!(~w(base64), input: [<<1, 2,>>, [3], [<<4, 5>>]])\n...> |> Enum.into(\"\")\n\"AQIDBAU=\n\"\n```\n\nWhen program exit abnormally it will raise\n`ExCmd.Stream.AbnormalExit` error with exit_status.\n\n```\niex> try do\n...>   ExCmd.stream!([\"sh\", \"-c\", \"exit 5\"]) |> Enum.to_list()\n...> rescue\n...>   e in ExCmd.Stream.AbnormalExit ->\n...>    e.exit_status\n...> end\n5\n```","ref":"ExCmd.html#stream!/2-examples"},{"type":"type","title":"ExCmd.collectable_func/0","doc":"","ref":"ExCmd.html#t:collectable_func/0"},{"type":"module","title":"ExCmd.Process","doc":"Server to interact with external process\n\n`ExCmd.stream!` should be preferred over this. Use this only if you need more control over the life-cycle of IO streams and OS process.","ref":"ExCmd.Process.html"},{"type":"function","title":"ExCmd.Process.await_exit/2","doc":"Waits for the program to terminate.\n\nIf the program terminates before timeout, it returns `{:ok, exit_status}` else returns `:timeout`","ref":"ExCmd.Process.html#await_exit/2"},{"type":"function","title":"ExCmd.Process.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExCmd.Process.html#child_spec/1"},{"type":"function","title":"ExCmd.Process.close_stdin/1","doc":"Closes input stream. Which signal EOF to the program","ref":"ExCmd.Process.html#close_stdin/1"},{"type":"function","title":"ExCmd.Process.os_pid/1","doc":"Returns os pid of the command","ref":"ExCmd.Process.html#os_pid/1"},{"type":"function","title":"ExCmd.Process.port_info/1","doc":"Returns [port_info](http://erlang.org/doc/man/erlang.html#port_info-1)","ref":"ExCmd.Process.html#port_info/1"},{"type":"function","title":"ExCmd.Process.read/2","doc":"Return bytes written by the program to output stream.\n\nThis blocks until the programs write and flush the output","ref":"ExCmd.Process.html#read/2"},{"type":"function","title":"ExCmd.Process.start_link/2","doc":"Starts a process using `cmd_with_args` and with options `opts`\n\n`cmd_with_args` must be a list containing command with arguments. example: `[\"cat\", \"file.txt\"]`.","ref":"ExCmd.Process.html#start_link/2"},{"type":"function","title":"Options - ExCmd.Process.start_link/2","doc":"* `cd`             -  the directory to run the command in\n  * `env`            -  a list of tuples containing environment key-value. These can be accessed in the external program\n  * `log`            -  When set to `true` odu logs and command stderr output are logged. Defaults to `false`","ref":"ExCmd.Process.html#start_link/2-options"},{"type":"function","title":"ExCmd.Process.status/1","doc":"Returns status of the process. It will be either of `:started`, `{:done, exit_status}`","ref":"ExCmd.Process.html#status/1"},{"type":"function","title":"ExCmd.Process.stop/1","doc":"Kills the program","ref":"ExCmd.Process.html#stop/1"},{"type":"function","title":"ExCmd.Process.write/3","doc":"Writes iodata `data` to programs input streams\n\nThis blocks when the pipe is full","ref":"ExCmd.Process.html#write/3"},{"type":"module","title":"ExCmd.Stream","doc":"Defines a `ExCmd.Stream` struct returned by `ExCmd.stream!/2`.","ref":"ExCmd.Stream.html"},{"type":"type","title":"ExCmd.Stream.t/0","doc":"","ref":"ExCmd.Stream.html#t:t/0"},{"type":"exception","title":"ExCmd.Process.Error","doc":"","ref":"ExCmd.Process.Error.html"},{"type":"exception","title":"ExCmd.Stream.AbnormalExit","doc":"","ref":"ExCmd.Stream.AbnormalExit.html"},{"type":"extras","title":"ExCmd","doc":"# ExCmd\n\n[![CI](https://github.com/akash-akya/ex_cmd/actions/workflows/elixir.yml/badge.svg)](https://github.com/akash-akya/ex_cmd/actions/workflows/elixir.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_cmd.svg)](https://hex.pm/packages/ex_cmd)\n[![docs](https://img.shields.io/badge/docs-hexpm-blue.svg)](https://hexdocs.pm/ex_cmd/)\n\n\nExCmd is an Elixir library to run and communicate with external\nprograms with back-pressure mechanism. It makes use os backed stdio\nbuffer for this.\n\nCommunication with external program using\n[Port](https://hexdocs.pm/elixir/Port.html) is not demand driven. So\nit is easy to run into memory issues when the size of the data we are\nwriting or reading from the external program is large. ExCmd tries to\nsolve this problem by making better use of os backed stdio buffers\nand providing demand-driven interface to write and read from external\nprogram. It can be used to stream data through an external\nprogram. For example, streaming a video through `ffmpeg` to serve a\nweb request.\n\nGetting audio out of a video stream is as simple as\n\n``` elixir\nExCmd.stream!(~w(ffmpeg -i pipe:0 -f mp3 pipe:1), input: File.stream!(\"music_video.mkv\", [], 65336))\n|> Stream.into(File.stream!(\"music.mp3\"))\n|> Stream.run()\n```","ref":"readme.html"},{"type":"extras","title":"Major Features - ExCmd","doc":"* Unlike beam ports, ExCmd puts back pressure on the external program\n* Stream abstraction\n* No separate shim installation required\n* Ships pre-built binaries for MacOS, Windows, Linux\n* Proper program termination. No more zombie process\n* Ability to close stdin and wait for output (with ports one can not selectively close stdin)","ref":"readme.html#major-features"},{"type":"extras","title":"Examples - ExCmd","doc":"```elixir\nExCmd.stream!(~w(curl ifconfig.co))\n|> Enum.into(\"\")\n```\n\nBinary as input\n\n  ```elixir\n  ExCmd.stream!(~w(cat), input: \"Hello World\")\n  |> Enum.into(\"\")\n  # => \"Hello World\"\n  ```\n\n  ```elixir\n  ExCmd.stream!(~w(base64), input: <<1, 2, 3, 4, 5>>)\n  |> Enum.into(\"\")\n  # => \"AQIDBAU=\\n\"\n  ```\n\nList of binary as input\n\n  ```elixir\n  ExCmd.stream!(~w(cat), input: [\"Hello \", \"World\"])\n  |> Enum.into(\"\")\n  # => \"Hello World\"\n  ```\n\niodata as input\n\n  ```elixir\n  ExCmd.stream!(~w(base64), input: [<<1, 2,>>, [3], [<<4, 5>>]])\n  |> Enum.into(\"\")\n  # => \"AQIDBAU=\\n\"\n  ```\n\nIf you want pipes and globs, you can spawn shell process and pass your\npipeline as argument\n\n```elixir\ncmd = \"echo 'foo bar' | base64\"\nExCmd.stream!([\"sh\", \"-c\", cmd])\n|> Enum.into(\"\")\n# => \"Zm9vIGJhcgo=\\n\"\n```\n\nRead [stream documentation](file:///Users/akash/repo/elixir/ex_cmd/doc/ExCmd.html#stream!/2) for information\nabout parameters.\n\n**Check out [Exile](https://github.com/akash-akya/exile) which is an\nalternative solution based on NIF without middleware overhead**","ref":"readme.html#examples"},{"type":"extras","title":"Installation - ExCmd","doc":"```elixir\ndef deps do\n  [\n    {:ex_cmd, \"~> x.x.x\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2020 Akash Hiremath\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}